--[[
Created by Grid2 original authors, modified by Michael
--]]

local L = LibStub("AceLocale-3.0"):GetLocale("Grid2Options")
local media = LibStub("LibSharedMedia-3.0", true)

function Grid2Options:MakeFrameOptions(reset)
local options= {
		mouseoverHighlight = {
			type = "toggle",
			name = L["Mouseover Highlight"],
			desc = L["Toggle mouseover highlight."],
			order = 59,
			get = function ()
				return Grid2Frame.db.profile.mouseoverHighlight
			end,
			set = function (_, v)
				Grid2Frame.db.profile.mouseoverHighlight = v
				Grid2Frame:WithAllFrames(function(f)
					f:EnableMouseoverHighlight(v)
				end)
			end,
		},
		tooltip = {
			type = "select",
			order = 55,
			name = L["Show Tooltip"],
			desc = L["Show unit tooltip.  Choose 'Always', 'Never', or 'OOC'."],
			get = function ()
				return Grid2Frame.db.profile.showTooltip
			end,
			set = function (_, v)
				Grid2Frame.db.profile.showTooltip = v
			end,
			values={["Always"] = L["Always"], ["Never"] = L["Never"], ["OOC"] = L["OOC"]},
		},
		frameBorder = {
			type = "range",
			order = 20,
			name = L["Border Size"],
			desc = L["Adjust the border of each unit's frame."],
			min = 1,
			max = 8,
			step = 1,
			get = function ()
				return Grid2Frame.db.profile.frameBorder
			end,
			set = function (_, frameBorder)
				Grid2Frame.db.profile.frameBorder = frameBorder
				Grid2Frame:WithAllFrames(function (f)
					f:SetBackdrop({
						bgFile = "Interface\\Addons\\Grid2\\white16x16", tile = true, tileSize = 16,
						edgeFile = "Interface\\Addons\\Grid2\\white16x16", edgeSize = frameBorder,
						insets = {left = frameBorder, right = frameBorder, top = frameBorder, bottom = frameBorder},
					})
					f:LayoutIndicators()
				end)
			end,
			disabled = InCombatLockdown,
		},
		framewidth = {
			type = "range",
			order = 30,
			name = L["Frame Width"],
			desc = L["Adjust the width of each unit's frame."],
			min = 10,
			max = 100,
			step = 1,
			get = function ()
				return Grid2Frame.db.profile.frameWidth
			end,
			set = function (_, v)
				Grid2Frame.db.profile.frameWidth = v
				Grid2Frame:ResizeAllFrames()
				Grid2Layout:UpdateHeadersSize()
				Grid2Layout:UpdateSize()
			end,
			disabled = InCombatLockdown,
		},
		frameheight = {
			type = "range",
			order = 40,
			name = L["Frame Height"],
			desc = L["Adjust the height of each unit's frame."],
			min = 10,
			max = 100,
			step = 1,
			get = function ()
				return Grid2Frame.db.profile.frameHeight
			end,
			set = function (_, v)
				Grid2Frame.db.profile.frameHeight = v
				Grid2Frame:ResizeAllFrames()
				Grid2Layout:UpdateHeadersSize()
				Grid2Layout:UpdateSize()
			end,
			disabled = InCombatLockdown,
		},
		orientation = {
			type = "select",
			order = 5,
			name = L["Orientation of Frame"],
			desc = L["Set frame orientation."],
			get = function ()
				return Grid2Frame.db.profile.orientation
			end,
			set = function (_, v)
				Grid2Frame.db.profile.orientation = v
				for _, indicator in Grid2:IterateIndicators() do
					if indicator.SetOrientation and indicator.orientation==nil then
						Grid2Frame:WithAllFrames(function (f) indicator:SetOrientation(f) end)
					end
				end
			end,
			values={["VERTICAL"] = L["VERTICAL"], ["HORIZONTAL"] = L["HORIZONTAL"]}
		},
		color = {
			type = "color",
			order = 58,
			name = L["Frame Background Color"],
			desc = L["Sets the background color of each grid2 frame"],
			get = function()
				local c= Grid2Frame.db.profile.frameColor
				if c then 
					return c.r, c.g, c.b, c.a
				else 
					return 0,0,0,1
				end
			end,
			set = function( info, r,g,b,a )
				local c= Grid2Frame.db.profile.frameColor
				if not c then
					c= {}
					Grid2Frame.db.profile.frameColor= c
				end	
				c.r, c.g, c.b, c.a = r, g, b, a
				Grid2Frame:WithAllFrames(function (f) f:SetBackgroundColor(r, g, b, a) end)
				Grid2Frame:ResetAllFrames()
				Grid2Frame:UpdateAllFrames()
			 end, 
			hasAlpha = true,
		}		
	}
	if Grid2Options.AddMediaOption then
		local textureOption = {
			type = "select",
			order = 54,
			name = L["Background Texture"],
			desc = L["Select the frame background texture."],
			get = function (info)
				local v = Grid2Frame.db.profile.frameTexture
				for i, t in ipairs(info.option.values) do
					if v == t then return i end
				end
			end,
			set = function (info, v)
				v = info.option.values[v]
				Grid2Frame.db.profile.frameTexture = v
				local texture = media:Fetch("statusbar", v)
				Grid2Frame:WithAllFrames(function (f) f:SetBackgroundTexture(texture) end)
				Grid2Frame:ResetAllFrames()
				Grid2Frame:UpdateAllFrames()		
			end,
		}
		Grid2Options:AddMediaOption("statusbar", textureOption)
		options.texture = textureOption
	end
	
	Grid2Options:AddModuleOptions("General", "Frames", options)
	
end